﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Collections.ObjectModel;

namespace BindingDemo
{
    public partial class MainWindow : Window
    {
        private DataGridScrollSynchronizer ScrollSync;

        private ObservableCollection<TestClass> items = new ObservableCollection<TestClass>();
        public ObservableCollection<TestClass> Items { get { return items; } }

        public MainWindow()
        {
            InitializeComponent();
		    this.DataContext = this;

		    for (int i = 0; i < 30; i++)
		    {
			    var d = new TestClass();
			    d.Data1 = i * 10 + 1;
			    d.Data2 = i * 10 + 2;
			    d.Data3 = i * 10 + 3;
			    d.Data4 = i * 10 + 4;
			    d.Data5 = i * 10 + 5;
			    Items.Add(d);
            }
		}

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            ScrollSync = new DataGridScrollSynchronizer(dataGrid1, dataGrid2);
            ScrollSync.Direction = SynchronizeDirection.Vertical;
        }
    }

    public class TestClass
    {
	    public int Data1 { get; set; }
	    public int Data2 { get; set; }
	    public int Data3 { get; set; }
	    public int Data4 { get; set; }
	    public int Data5 { get; set; }

	    public TestClass()
	    {
	    }
    }

    [Flags]
    public enum SynchronizeDirection
    {
	    Horizontal = 0x01,
	    Vertical = 0x02,
	    Both = 0x03,
    }


    public class DataGridScrollSynchronizer
    {
	    private ScrollViewer sv1;
	    private ScrollViewer sv2;

	    public SynchronizeDirection Direction { get; set; }

	    public DataGridScrollSynchronizer(DataGrid dg1, DataGrid dg2)
	    {
		    sv1 = GetScrollViewer(dg1);
		    sv2 = GetScrollViewer(dg2);

		    sv1.ScrollChanged += sv1_ScrollChanged;
		    sv2.ScrollChanged += sv2_ScrollChanged;
	    }

	    private ScrollViewer GetScrollViewer(FrameworkElement fe)
	    {
		    int n = VisualTreeHelper.GetChildrenCount(fe);
		    for (int i = 0; i < n; i++)
		    {
			    var child = VisualTreeHelper.GetChild(fe, i) as FrameworkElement;
			    if (child == null)
			    {
				    continue;
			    }

			    if (child is ScrollViewer)
			    {
				    return (ScrollViewer)child;
			    }

			    var sv = GetScrollViewer(child);
			    if (sv != null)
			    {
				    return sv;
			    }
		    }
		    return null;
	    }

	    private void sv1_ScrollChanged(object sender, ScrollChangedEventArgs e)
	    {
		    if (Direction.HasFlag(SynchronizeDirection.Horizontal))
		    {
			    sv2.ScrollToHorizontalOffset(sv1.HorizontalOffset);
		    }
		    if (Direction.HasFlag(SynchronizeDirection.Vertical))
		    {
			    sv2.ScrollToVerticalOffset(sv1.VerticalOffset);
		    }
	    }

	    private void sv2_ScrollChanged(object sender, ScrollChangedEventArgs e)
	    {
		    if (Direction.HasFlag(SynchronizeDirection.Horizontal))
		    {
			    sv1.ScrollToHorizontalOffset(sv2.HorizontalOffset);
		    }
		    if (Direction.HasFlag(SynchronizeDirection.Vertical))
		    {
			    sv1.ScrollToVerticalOffset(sv2.VerticalOffset);
		    }
	    }
    }
}



